
function [ts,ys]=forward_euler(f,y0,t0,tmax,dt)
ts=t0:dt:tmax;    
P=size(y0,1);    
K=size(ts,2);
ys=zeros(P,K);
ys(:,1)=y0;
for k=1:K-1
    //disp(f(ts(k),ys(:,k)))
    ys(:,k+1)=ys(:,k) + dt*f(ts(k),ys(:,k));    
end   
end

function [ts,ys]=RK2(f,y0,t0,tmax,dt)
ts=t0:dt:tmax;    
P=size(y0,1);    
K=size(ts,2);
ys=zeros(P,K);
ys(:,1)=y0;
for k=1:K-1
    ykm = ys(:,k) + dt/2*f(ts(k),ys(:,k));      
    ys(:,k+1)=ys(:,k) + dt*f(ts(k)+dt/2,ykm);
end 
end


function [ts,ys,nits]=backward_euler(f,dfdy,y0,t0,tmax,dt,tol)
ts=t0:dt:tmax;    
P=size(y0,1);    
K=size(ts,2);
ys=zeros(P,K);
nits=zeros(K,1);
ys(:,1)=y0;
for k=1:K-1
    [ys(:,k+1),nits(k+1)] = newton_be(f,dfdy,ys(:,k),ts(k+1),dt,tol);
end   
end



function [ykp1,nits] = newton_be(f,dfdy,yk,tkp1,dt,tol)
nits=0;  
ykp1=yk;
P=size(yk,1);
maxit=100;
while nits<=maxit
    fk   = ykp1 - yk - dt*f(tkp1,ykp1);
    if norm(fk)<tol
        break;
    end
    dfkdy= eye(P,P) - dt*dfdy(tkp1,ykp1);
    ykp1= ykp1 - dfkdy\fk;   
    nits=nits+1;
end           
end
